@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nill; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2004-2005 Imendio AB
  * Copyright (C) 2002-2003 CodeFactory AB
@@ -1016,9 +1016,11 @@ old_xml_read_project (MrpParser *parser)
 				     GINT_TO_POINTER (MRP_DAY_USE_BASE),
 				     mrp_day_ref (mrp_day_get_use_base ()));
 		
-		/* Read day-types */
-		for (day = child->children; day; day = day->next) {
-			old_xml_read_day_type (parser, day);
+		if (child != NULL) {
+		        /* Read day-types */
+        		for (day = child->children; day; day = day->next) {
+        			old_xml_read_day_type (parser, day);
+        		}
 		}
 		
 		for (child = calendars->children; child; child = child->next) {
@@ -1499,6 +1501,8 @@ mrp_old_xml_parse (MrpProject *project, xmlDoc *doc, GError **error)
         task_manager = imrp_project_get_task_manager (project);
         mrp_task_manager_set_root (task_manager, parser.root_task);
 
+	parser.project_start = mrp_time_align_day (parser.project_start);
+	
 	g_object_set (project,
                       "project-start", parser.project_start,
                       "default-group", parser.default_group,
@@ -1527,8 +1531,6 @@ mrp_old_xml_parse (MrpProject *project, xmlDoc *doc, GError **error)
         g_list_free (parser.assignments);
         g_list_free (parser.resources);
 	
-/* 	module->resources = parser.resources; */
-	
 	return TRUE;
 }
 
