@@ -55,6 +55,7 @@ static gdouble f = 1.0;
 
 #define CRITICAL_PATH_KEY  "/views/gantt_view/highlight_critical_path"
 #define NOSTDDAYS_PATH_KEY "/views/gantt_view/display_nonstandard_days"
+#define GUIDELINES_PATH_KEY "/views/gantt_view/show_guidelines"
 
 
 typedef struct _TreeNode TreeNode; 
@@ -108,6 +109,9 @@ struct _PlannerGanttChartPriv {
 	/* Nonstandard days visualization */
 	gboolean         nonstandard_days;
 
+	/* GuideLines . */
+	gboolean 		 guidelines;
+
 	/* Keep a list of signal connection ids, so we can remove them
 	 * easily.
 	 */
@@ -177,7 +181,6 @@ static void        gantt_chart_task_removed             (MrpTask            *tas
 static void        gantt_chart_build_tree               (PlannerGanttChart  *chart);
 static void        gantt_chart_reflow                   (PlannerGanttChart  *chart,
 							 gboolean            height_changed);
-static void        gantt_chart_reflow_now               (PlannerGanttChart  *chart);
 static TreeNode *  gantt_chart_insert_task              (PlannerGanttChart  *chart,
 							 GtkTreePath        *path,
 							 MrpTask            *task);
@@ -186,6 +189,11 @@ gantt_chart_add_relation                                (PlannerGanttChart  *cha
 							 TreeNode           *task,
 							 TreeNode           *predecessor,
 							 MrpRelationType     type);
+static void        gantt_chart_get_visible_region       (PlannerGanttChart *chart,
+							 gdouble            *x1,
+							 gdouble            *y1,
+							 gdouble            *x2,
+							 gdouble            *y2);
 static void        gantt_chart_set_scroll_region        (PlannerGanttChart  *chart,
 							 gdouble             x1,
 							 gdouble             y1,
@@ -390,6 +398,12 @@ gantt_chart_init (PlannerGanttChart *chart)
 							  NULL);
 	priv->nonstandard_days   = planner_conf_get_bool (NOSTDDAYS_PATH_KEY,
 							  NULL);
+	priv->guidelines = planner_conf_get_bool (GUIDELINES_PATH_KEY, NULL);
+
+	g_object_set (G_OBJECT(chart->priv->background),
+			"show_guidelines",
+			priv->guidelines,
+			NULL);
 }
 
 static void
@@ -562,7 +576,7 @@ gantt_chart_map (GtkWidget *widget)
 	}
 
 	chart->priv->height_changed = TRUE;
-	gantt_chart_reflow_now (chart);
+	planner_gantt_chart_reflow_now (chart);
 }
 
 static void
@@ -582,7 +596,7 @@ gantt_chart_size_allocate (GtkWidget     *widget,
 	 * jumping around.
 	 */
 	if (GTK_WIDGET_MAPPED (chart)) {
-		gantt_chart_reflow_now (chart);
+		planner_gantt_chart_reflow_now (chart);
 	}
 }
 
@@ -953,6 +967,7 @@ gantt_chart_reflow_idle (PlannerGanttChart *chart)
 	PlannerGanttChartPriv *priv;
 	mrptime                t1, t2;
 	gdouble                x1, y1, x2, y2;
+	gdouble                vx1, vy1, vx2, vy2;
 	gdouble                width, height;
 	gdouble                bx1, bx2;
 	GtkAllocation          allocation;
@@ -991,11 +1006,16 @@ gantt_chart_reflow_idle (PlannerGanttChart *chart)
 
 	/* Put some padding after the right-most coordinate. */
 	bx2 += PADDING;
-		
+
 	width = MAX (width, bx2 - bx1);
 
 	x2 = x1 + width;
-	
+
+	gantt_chart_get_visible_region(chart, &vx1, &vy1, &vx2, &vy2);
+
+	x2 = MAX (x2, vx2);
+	y2 = MAX (y2, vy2);
+
 	gantt_chart_set_scroll_region (chart,
 				       x1,
 				       y1,
@@ -1015,8 +1035,8 @@ gantt_chart_reflow_idle (PlannerGanttChart *chart)
 	return FALSE;
 }
 
-static void
-gantt_chart_reflow_now (PlannerGanttChart *chart)
+void
+planner_gantt_chart_reflow_now (PlannerGanttChart *chart)
 {
 	if (!GTK_WIDGET_MAPPED (chart)) {
 		return;
@@ -1189,7 +1209,7 @@ gantt_chart_project_start_changed (MrpProject   *project,
 		      "project-start", t,
 		      NULL);
 	
-	gantt_chart_reflow_now (chart);
+	planner_gantt_chart_reflow_now (chart);
 }
 
 static void
@@ -1511,7 +1531,7 @@ planner_gantt_chart_set_model (PlannerGanttChart *chart,
 		 * start-up .
 		 */
 		priv->height_changed = TRUE;
-		gantt_chart_reflow_now (chart);
+		planner_gantt_chart_reflow_now (chart);
 	}
 	
 	g_object_notify (G_OBJECT (chart), "model");
@@ -1650,6 +1670,25 @@ gantt_chart_tree_traverse (TreeNode *node, TreeFunc func, gpointer data)
 	func (node, data);
 }
 
+static void
+gantt_chart_get_visible_region (PlannerGanttChart *chart,
+			       gdouble            *x1,
+			       gdouble            *y1,
+			       gdouble            *x2,
+			       gdouble            *y2)
+{
+	GnomeCanvas *canvas;
+	gint cx, cy;
+	
+	canvas = chart->priv->canvas;
+
+	gnome_canvas_get_scroll_offsets(canvas, &cx, &cy);
+	gnome_canvas_c2w(canvas, cx, cy, x1, y1);
+
+	*x2 = *x1 + GTK_WIDGET(canvas)->allocation.width;
+	*y2 = *y1 + GTK_WIDGET(canvas)->allocation.height;
+}
+
 static void
 gantt_chart_set_scroll_region (PlannerGanttChart *chart,
 			       gdouble            x1,
@@ -1740,7 +1779,7 @@ gantt_chart_set_zoom (PlannerGanttChart *chart, gdouble zoom)
 			       "zoom", priv->zoom,
 			       NULL);
 
-	gantt_chart_reflow_now (chart);
+	planner_gantt_chart_reflow_now (chart);
 }
 
 static mrptime 
@@ -1929,12 +1968,28 @@ planner_gantt_chart_set_show_guidelines (PlannerGanttChart *chart,
 
 	priv = chart->priv;
 
+	if (priv->guidelines == state) {
+		return;
+	}
+
+	priv->guidelines = state;
+
 	g_object_set (G_OBJECT(chart->priv->background),
 		      "show_guidelines",
 		      state,
 		      NULL);
 
 	gtk_widget_queue_draw (GTK_WIDGET (priv->canvas));
+	
+	planner_conf_set_bool (GUIDELINES_PATH_KEY, state, NULL);
+}
+
+gboolean
+planner_gantt_chart_get_show_guidelines (PlannerGanttChart *chart)
+{
+	g_return_val_if_fail (PLANNER_IS_GANTT_CHART (chart), FALSE);
+
+	return chart->priv->guidelines;
 }
 
 gboolean
